# -*- coding: utf-8 -*-

# Copyright (C) 2004-2005 Johan Svedberg <johan@svedberg.com>

# This file is part of ontv.

# ontv is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# ontv is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with ontv; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import cPickle
import os.path
import thread

from XMLTVFile import XMLTVFile

class Listings(dict):
    def __init__(self, config):
        self.config = config
        self.file = os.path.join(config.base_dir, "listings.p")
        self.programs = []
        self.selected_channels = []

        self.xmltvfile = XMLTVFile(self)

    def __getstate__(self):
        return (self.copy(), self.__dict__.copy())

    def __setstate__(self, (listings, attributes)):
        self.update(listings)
        self.__dict__.update(attributes)

    def add_channel(self, channel):
        self[channel.name] = channel

    def add_program(self, program):
        self.programs.append(program)
        self[program.channel_name].programs.append(program)

    def __save_in_thread(self):
        file = open(self.file, 'w')
        cPickle.dump(self, file, cPickle.HIGHEST_PROTOCOL)
        file.close()
        if self.config.debug:
            print("Saved listings file: %s" % self.file)

    def save(self):
        thread.start_new_thread(self.__save_in_thread, ())

    def load(self, force_reload=False):
        if os.path.exists(self.file):
            file = open(self.file)
            self = cPickle.load(file)
            file.close()
            if self.config.debug:
                print("Loaded listings from file: %s" % self.file)
        if self.xmltvfile.has_changed() or force_reload:
            self.xmltvfile.load()
            if self.config.debug:
                print("Loaded listings from XMLTV file: %s" % self.xmltvfile.path)
        return (self, self.__dict__)

    def search_program(self, search_terms):
        matches = []
        for term in search_terms:
            for program in self.programs:
                if term in program.title.lower():
                    matches.append(program)
        return matches

# vim: set sw=4 et sts=4 tw=79 fo+=l:
