# -*- coding: utf-8 -*-

# Copyright (C) 2004-2005 Johan Svedberg <johan@svedberg.com>

# This file is part of ontv.

# ontv is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# ontv is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with ontv; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import gettext
import locale
import os.path
import sys
import thread
from gettext import gettext as _

import pygtk
pygtk.require("2.0")
import gtk
import gnome
import gnomeapplet
import gobject

from Configuration import Configuration
from Listings import Listings
from ProgramWindow import ProgramWindow
from ToggleButton import ToggleButton
from PreferencesDialog import PreferencesDialog
from SearchDialog import SearchDialog
from AboutDialog import AboutDialog
from ontv import NAME, VERSION, UI_DIR, LOCALE_DIR

class OnTVApplet:
    def __init__(self, applet):
        self.applet = applet
        self.auto_reload_id = None
        gtk.threads_init()

        self.config = Configuration()
        self.listings = Listings(self.config)

        self.pw = ProgramWindow(self.listings)
        self.tb = ToggleButton(self.config, self.pw)
        self.pd = PreferencesDialog(self)

        self.sd = SearchDialog(self.listings)

        ui_file = os.path.join(UI_DIR, "GNOME_OnTVApplet.xml")
        verb_list = [("Reload XMLTV file", self.load_listings),
                     ("Preferences", self.pd.show),
                     ("Search Program", self.sd.show),
                     ("About", self.__run_ad)]
        self.applet.setup_menu_from_file(None, ui_file, NAME, verb_list)

        self.applet.add(self.tb)

        self.applet.show_all()

    def load_listings(self, uicomponent=None, verb=None):
        thread.start_new_thread(self.__load_listings_in_thread, ())

    def __load_listings_in_thread(self):
        self.tb.set_tooltip(_("Loading XMLTV file..."))
        listings = self.listings.load()
        self.listings.update(listings[0])
        self.listings.__dict__.update(listings[1])
        gtk.threads_enter()
        self.pw.update()
        self.pd.update_channels_treeview()
        gtk.threads_leave()
        self.tb.set_tooltip(None)

    def __run_ad(self, uicomponent=None, verb=None):
        self.ad = AboutDialog()
        self.ad.run()

    def start_auto_reload(self, rate):
        if rate >= 1:
            timeout = rate * 3600 * 1000
            self.auto_reload_id = gobject.timeout_add(timeout,
                                                      self.load_listings)
            if self.config.debug:
                print("[Added timeout %s]: %sh" % (str(self.auto_reload_id),
                                                   str(rate)))

    def stop_auto_reload(self):
        if self.auto_reload_id:
            gobject.source_remove(self.auto_reload_id)
            if self.config.debug:
                print("[Removed timeout %s]" % (str(self.auto_reload_id)))

    def run(self):
        if self.config.xmltv_file == '':
            self.pd.show()
            self.pd.notebook.set_current_page(0)
        else:
            self.listings.xmltvfile.path = self.config.xmltv_file
            if self.config.auto_reload_xmltv_file:
                self.start_auto_reload(self.config.auto_reload_xmltv_file_rate)
            self.load_listings()

def main():
    gettext.bindtextdomain(NAME.lower(), LOCALE_DIR)
    gettext.textdomain(NAME.lower())

    locale.bindtextdomain(NAME.lower(), LOCALE_DIR)
    locale.textdomain(NAME.lower())

    gnome.init(NAME, VERSION)

    if len(sys.argv) > 1 and sys.argv[1] == "run-in-window":
        window = gtk.Window()
        window.set_title(NAME)
        window.connect("destroy", gtk.main_quit)
        applet = gnomeapplet.Applet()
        applet_factory(applet)
        applet.reparent(window)
        window.show_all()
        gtk.main()
    else:
        activate_factory()

def applet_factory(applet, iid=None):
    ontvapplet = OnTVApplet(applet)
    ontvapplet.run()
    return True

def activate_factory():
    gnomeapplet.bonobo_factory("OAFIID:GNOME_OnTVApplet_Factory",
                                gnomeapplet.Applet.__gtype__, NAME, VERSION,
                                applet_factory)

# vim: set sw=4 et sts=4 tw=79 fo+=l:
