# -*- coding: utf-8 -*-

# Copyright (C) 2004-2005 Johan Svedberg <johan@svedberg.com>

# This file is part of ontv.

# ontv is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# ontv is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with ontv; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import os.path
from gettext import gettext as _

import gtk
import gobject
import xmltv

from GUI import ErrorDialog
from Channel import Channel
from Program import Program

class XMLTVFile:
    def __init__(self, listings):
        self.listings = listings

        self.path = None
        self.mtime = None
        self.__channel_names = {}

    def has_changed(self):
        if os.path.exists(self.path):
            return os.path.getmtime(self.path) != self.mtime
        return False

    def load(self):
        title = error_msg = ""
        try:
            self.__load_channels()
            self.__load_programs()
            self.mtime = os.path.getmtime(self.path)
        except IOError, ioe:
            title = _("Failed to load ") + self.path
            if ioe.errno == 2:
                error_msg = _("File not found")
            elif ioe.errno == 13:
                error_msg = _("Access denied")
            else:
                error_msg = _("Unknown error")
        # except Exception, e:
            # if e.lineno and e.offset:
                # title = _("Error while parsing ") + self.path
                # error_msg = _("Not well formed at line %s, column %s") % (e.lineno, e.offset)

        if title and error_msg:
            gtk.threads_enter()
            ed = ErrorDialog(title, error_msg)
            ed.run()
            ed.destroy()
            gtk.threads_leave()
            return

        for channel in self.listings.values():
            channel.programs.sort()

        self.listings.save()

    def __load_channels(self):
        for channel in xmltv.read_channels(open(self.path)):
            c = Channel(channel)
            self.__channel_names[c.id] = c.name
            self.listings.add_channel(c)

    def __load_programs(self):
        for program in xmltv.read_programmes(open(self.path)):
            channel_name = self.__get_channel_name(program)
            self.listings.add_program(Program(program, channel_name))

    def __get_channel_name(self, program):
        return self.__channel_names[program["channel"]]

# vim: set sw=4 et sts=4 tw=79 fo+=l:
